#!/usr/bin/env bash

# The command of Python
export PYTHON=python3

# The home directory of virtual environment
export VIRTUAL_ENV=${VIRTUAL_ENV}

# The directory where configuration files are stored
export GUNIFLASK_CONF_DIR=${GUNIFLASK_CONF_DIR}

# The directory where log files are stored
export GUNIFLASK_LOG_DIR=${GUNIFLASK_LOG_DIR}

# The directory where pid files are stored
export GUNIFLASK_PID_DIR=${GUNIFLASK_PID_DIR}

# A string representing this instance
export GUNIFLASK_ID_STRING=${GUNIFLASK_ID_STRING}
